@@ -19,14 +19,16 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef GTK_SOURCE_COMPLETION_PIVATE_H
-#define GTK_SOURCE_COMPLETION_PIVATE_H
+#ifndef __GTK_SOURCE_COMPLETION_PIVATE_H__
+#define __GTK_SOURCE_COMPLETION_PIVATE_H__
 
 #include <gtk/gtk.h>
 #include <gtksourceview/gtksourcetypes.h>
 
-G_GNUC_INTERNAL
-GtkSourceCompletion *gtk_source_completion_new (GtkSourceView *source_view);
+/* Forward declaration */
+struct _GtkSourceView;
 
-#endif /* GTK_SOURCE_COMPLETION_PIVATE_H */
+GtkSourceCompletion *gtk_source_completion_new (struct _GtkSourceView *source_view);
+
+#endif /* __GTK_SOURCE_COMPLETION_PIVATE_H__ */
 
