@@ -20,14 +20,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gtksourcemarkattributes.h"
-#include "gtksourcemark.h"
 #include "gtksourceview-i18n.h"
 #include "gtksourcepixbufhelper.h"
+#include "gtksourceview-marshal.h"
 
 /**
  * SECTION:markattributes
@@ -35,8 +31,8 @@
  * @title: GtkSourceMarkAttributes
  * @see_also: #GtkSourceMark
  *
- * #GtkSourceMarkAttributes is an object specifying attributes used by
- * a #GtkSourceView to visually show lines marked with #GtkSourceMark<!-- -->s
+ * #GtkSourceMarkAttribute<!-- -->s is an object specifying attributes used by
+ * a GtkSourceView to visually show lines marked with #GtkSourceMark<!-- -->s
  * of a specific category. It allows you to define a background color of a line,
  * an icon shown in gutter and tooltips.
  *
@@ -48,7 +44,7 @@
  * An icon is a graphic element which is shown in the gutter of a view. An
  * example use is showing a red filled circle in a debugger to show that a
  * breakpoint was set in certain line. To get an icon that will be placed in
- * a gutter, first a base for it must be specified and then
+ * a gutter first a base for it must be specified and then
  * gtk_source_mark_attributes_render_icon() must be called.
  * There are several ways to specify a base for an icon:
  * <itemizedlist>
@@ -74,15 +70,16 @@
  *  </listitem>
  * </itemizedlist>
  * Using any of the above functions overrides the one used earlier. But note
- * that a getter counterpart of earlier used function can still return some
+ * that a getter counterpart of ealier used function can still return some
  * value, but it is just not used when rendering the proper icon.
  *
  * To provide meaningful tooltips for a given mark of a category, you should
- * connect to #GtkSourceMarkAttributes::query-tooltip-text or
- * #GtkSourceMarkAttributes::query-tooltip-markup where the latter
+ * connect to ::query-tooltip-text or ::query-tooltip-markup where the latter
  * takes precedence.
  */
 
+#define GTK_SOURCE_MARK_ATTRIBUTES_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_MARK_ATTRIBUTES, GtkSourceMarkAttributesPrivate))
+
 struct _GtkSourceMarkAttributesPrivate
 {
 	GdkRGBA background;
@@ -92,7 +89,7 @@ struct _GtkSourceMarkAttributesPrivate
 	guint background_set : 1;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceMarkAttributes, gtk_source_mark_attributes, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GtkSourceMarkAttributes, gtk_source_mark_attributes, G_TYPE_OBJECT)
 
 enum
 {
@@ -108,10 +105,10 @@ enum
 {
 	QUERY_TOOLTIP_TEXT,
 	QUERY_TOOLTIP_MARKUP,
-	N_SIGNALS
+	NUM_SIGNALS
 };
 
-static guint signals[N_SIGNALS];
+static guint signals[NUM_SIGNALS] = {0,};
 
 static void
 gtk_source_mark_attributes_finalize (GObject *object)
@@ -125,7 +122,7 @@ gtk_source_mark_attributes_finalize (GObject *object)
 
 static void
 set_background (GtkSourceMarkAttributes *attributes,
-		const GdkRGBA           *color)
+                const GdkRGBA         *color)
 {
 	if (color)
 	{
@@ -139,7 +136,7 @@ set_background (GtkSourceMarkAttributes *attributes,
 
 static void
 set_stock_id (GtkSourceMarkAttributes *attributes,
-	      const gchar             *stock_id)
+              const gchar           *stock_id)
 {
 	if (0 != g_strcmp0 (gtk_source_pixbuf_helper_get_stock_id (attributes->priv->helper),
 	                                                           stock_id))
@@ -153,7 +150,7 @@ set_stock_id (GtkSourceMarkAttributes *attributes,
 
 static void
 set_icon_name (GtkSourceMarkAttributes *attributes,
-	       const gchar             *icon_name)
+               const gchar           *icon_name)
 {
 	if (g_strcmp0 (gtk_source_pixbuf_helper_get_icon_name (attributes->priv->helper),
 	                                                       icon_name) == 0)
@@ -169,7 +166,7 @@ set_icon_name (GtkSourceMarkAttributes *attributes,
 
 static void
 set_pixbuf (GtkSourceMarkAttributes *attributes,
-	    const GdkPixbuf         *pixbuf)
+            const GdkPixbuf       *pixbuf)
 {
 	if (gtk_source_pixbuf_helper_get_pixbuf (attributes->priv->helper) == pixbuf)
 	{
@@ -184,7 +181,7 @@ set_pixbuf (GtkSourceMarkAttributes *attributes,
 
 static void
 set_gicon (GtkSourceMarkAttributes *attributes,
-	   GIcon                   *gicon)
+           GIcon                 *gicon)
 {
 	if (gtk_source_pixbuf_helper_get_gicon (attributes->priv->helper) == gicon)
 	{
@@ -199,9 +196,9 @@ set_gicon (GtkSourceMarkAttributes *attributes,
 
 static void
 gtk_source_mark_attributes_set_property (GObject      *object,
-					 guint         prop_id,
-					 const GValue *value,
-					 GParamSpec   *pspec)
+                                         guint         prop_id,
+                                         const GValue *value,
+                                         GParamSpec   *pspec)
 {
 	GtkSourceMarkAttributes *self = GTK_SOURCE_MARK_ATTRIBUTES (object);
 
@@ -230,9 +227,9 @@ gtk_source_mark_attributes_set_property (GObject      *object,
 
 static void
 gtk_source_mark_attributes_get_property (GObject    *object,
-					 guint       prop_id,
-					 GValue     *value,
-					 GParamSpec *pspec)
+                                         guint       prop_id,
+                                         GValue     *value,
+                                         GParamSpec *pspec)
 {
 	GtkSourceMarkAttributes *self = GTK_SOURCE_MARK_ATTRIBUTES (object);
 
@@ -280,6 +277,8 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	object_class->get_property = gtk_source_mark_attributes_get_property;
 	object_class->set_property = gtk_source_mark_attributes_set_property;
 
+	g_type_class_add_private (object_class, sizeof (GtkSourceMarkAttributesPrivate));
+
 	/**
 	 * GtkSourceMarkAttributes:background:
 	 *
@@ -288,28 +287,23 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_BACKGROUND,
 	                                 g_param_spec_boxed ("background",
-	                                                     "Background",
-	                                                     "The background",
+	                                                     _("Background"),
+	                                                     _("The background"),
 	                                                     GDK_TYPE_RGBA,
-	                                                     G_PARAM_READWRITE |
-							     G_PARAM_STATIC_STRINGS));
+	                                                     G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceMarkAttributes:stock-id:
 	 *
 	 * A stock id that may be a base of a rendered icon.
-	 *
-	 * Deprecated: 3.10: Don't use this property.
 	 */
 	g_object_class_install_property (object_class,
 	                                 PROP_STOCK_ID,
 	                                 g_param_spec_string ("stock-id",
-	                                                      "Stock Id",
-	                                                      "The stock id",
+	                                                      _("Stock Id"),
+	                                                      _("The stock id"),
 	                                                      NULL,
-	                                                      G_PARAM_READWRITE |
-							      G_PARAM_DEPRECATED |
-							      G_PARAM_STATIC_STRINGS));
+	                                                      G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceMarkAttributes:pixbuf:
@@ -319,11 +313,10 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_PIXBUF,
 	                                 g_param_spec_object ("pixbuf",
-	                                                      "Pixbuf",
-	                                                      "The pixbuf",
+	                                                      _("Pixbuf"),
+	                                                      _("The pixbuf"),
 	                                                      GDK_TYPE_PIXBUF,
-	                                                      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
+	                                                      G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceMarkAttributes:icon-name:
@@ -333,11 +326,10 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_ICON_NAME,
 	                                 g_param_spec_string ("icon-name",
-	                                                      "Icon Name",
-	                                                      "The icon name",
+	                                                      _("Icon Name"),
+	                                                      _("The icon name"),
 	                                                      NULL,
-	                                                      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
+	                                                      G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceMarkAttributes:gicon:
@@ -347,11 +339,10 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_GICON,
 	                                 g_param_spec_object ("gicon",
-	                                                      "GIcon",
-	                                                      "The GIcon",
+	                                                      _("GIcon"),
+	                                                      _("The GIcon"),
 	                                                      G_TYPE_ICON,
-	                                                      G_PARAM_READWRITE |
-							      G_PARAM_STATIC_STRINGS));
+	                                                      G_PARAM_READWRITE));
 
 	/**
 	 * GtkSourceMarkAttributes::query-tooltip-text:
@@ -369,7 +360,9 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 		              G_TYPE_FROM_CLASS (klass),
 		              G_SIGNAL_RUN_LAST,
 		              0,
-		              NULL, NULL, NULL,
+		              NULL,
+		              NULL,
+		              _gtksourceview_marshal_STRING__OBJECT,
 		              G_TYPE_STRING,
 		              1,
 		              GTK_SOURCE_TYPE_MARK);
@@ -390,7 +383,9 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 		              G_TYPE_FROM_CLASS (klass),
 		              G_SIGNAL_RUN_LAST,
 		              0,
-		              NULL, NULL, NULL,
+		              NULL,
+		              NULL,
+		              _gtksourceview_marshal_STRING__OBJECT,
 		              G_TYPE_STRING,
 		              1,
 		              GTK_SOURCE_TYPE_MARK);
@@ -399,7 +394,7 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 static void
 gtk_source_mark_attributes_init (GtkSourceMarkAttributes *self)
 {
-	self->priv = gtk_source_mark_attributes_get_instance_private (self);
+	self->priv = GTK_SOURCE_MARK_ATTRIBUTES_GET_PRIVATE (self);
 
 	self->priv->helper = gtk_source_pixbuf_helper_new ();
 }
@@ -412,7 +407,7 @@ gtk_source_mark_attributes_init (GtkSourceMarkAttributes *self)
  * Returns: (transfer full): a new source mark attributes.
  */
 GtkSourceMarkAttributes *
-gtk_source_mark_attributes_new (void)
+gtk_source_mark_attributes_new ()
 {
 	return g_object_new (GTK_SOURCE_TYPE_MARK_ATTRIBUTES, NULL);
 }
@@ -426,7 +421,7 @@ gtk_source_mark_attributes_new (void)
  */
 void
 gtk_source_mark_attributes_set_background (GtkSourceMarkAttributes *attributes,
-					   const GdkRGBA           *background)
+                                           const GdkRGBA           *background)
 {
 	g_return_if_fail (GTK_SOURCE_IS_MARK_ATTRIBUTES (attributes));
 
@@ -444,7 +439,7 @@ gtk_source_mark_attributes_set_background (GtkSourceMarkAttributes *attributes,
  */
 gboolean
 gtk_source_mark_attributes_get_background (GtkSourceMarkAttributes *attributes,
-					   GdkRGBA                 *background)
+                                           GdkRGBA                 *background)
 {
 	g_return_val_if_fail (GTK_SOURCE_IS_MARK_ATTRIBUTES (attributes), FALSE);
 
@@ -461,12 +456,10 @@ gtk_source_mark_attributes_get_background (GtkSourceMarkAttributes *attributes,
  * @stock_id: a stock id.
  *
  * Sets stock id to be used as a base for rendered icon.
- *
- * Deprecated: 3.10: Don't use this function.
  */
 void
 gtk_source_mark_attributes_set_stock_id (GtkSourceMarkAttributes *attributes,
-					 const gchar             *stock_id)
+                                         const gchar             *stock_id)
 {
 	g_return_if_fail (GTK_SOURCE_IS_MARK_ATTRIBUTES (attributes));
 
@@ -482,8 +475,6 @@ gtk_source_mark_attributes_set_stock_id (GtkSourceMarkAttributes *attributes,
  *
  * Returns: (transfer none): Stock id. Returned string is owned by @attributes and
  * shouldn't be freed.
- *
- * Deprecated: 3.10: Don't use this function.
  */
 const gchar *
 gtk_source_mark_attributes_get_stock_id (GtkSourceMarkAttributes *attributes)
@@ -502,7 +493,7 @@ gtk_source_mark_attributes_get_stock_id (GtkSourceMarkAttributes *attributes)
  */
 void
 gtk_source_mark_attributes_set_icon_name (GtkSourceMarkAttributes *attributes,
-					  const gchar             *icon_name)
+                                          const gchar             *icon_name)
 {
 	g_return_if_fail (GTK_SOURCE_IS_MARK_ATTRIBUTES (attributes));
 
@@ -536,7 +527,7 @@ gtk_source_mark_attributes_get_icon_name (GtkSourceMarkAttributes *attributes)
  */
 void
 gtk_source_mark_attributes_set_gicon (GtkSourceMarkAttributes *attributes,
-				      GIcon                   *gicon)
+                                      GIcon                   *gicon)
 {
 	g_return_if_fail (GTK_SOURCE_IS_MARK_ATTRIBUTES (attributes));
 
@@ -570,7 +561,7 @@ gtk_source_mark_attributes_get_gicon (GtkSourceMarkAttributes *attributes)
  */
 void
 gtk_source_mark_attributes_set_pixbuf (GtkSourceMarkAttributes *attributes,
-				       const GdkPixbuf         *pixbuf)
+                                       const GdkPixbuf         *pixbuf)
 {
 	g_return_if_fail (GTK_SOURCE_IS_MARK_ATTRIBUTES (attributes));
 
@@ -612,8 +603,8 @@ gtk_source_mark_attributes_get_pixbuf (GtkSourceMarkAttributes *attributes)
  */
 const GdkPixbuf *
 gtk_source_mark_attributes_render_icon (GtkSourceMarkAttributes *attributes,
-					GtkWidget               *widget,
-					gint                     size)
+                                        GtkWidget               *widget,
+                                        gint                     size)
 {
 	g_return_val_if_fail (GTK_SOURCE_IS_MARK_ATTRIBUTES (attributes), NULL);
 	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
@@ -630,7 +621,7 @@ gtk_source_mark_attributes_render_icon (GtkSourceMarkAttributes *attributes,
  * @mark: a #GtkSourceMark.
  *
  * Queries for a tooltip by emitting
- * a #GtkSourceMarkAttributes::query-tooltip-text signal. The tooltip is a plain
+ * a GtkSourceMarkAttributes::query-tooltip-text signal. The tooltip is a plain
  * text.
  *
  * Returns: (transfer full): A tooltip. The returned string should be freed by
@@ -638,7 +629,7 @@ gtk_source_mark_attributes_render_icon (GtkSourceMarkAttributes *attributes,
  */
 gchar *
 gtk_source_mark_attributes_get_tooltip_text (GtkSourceMarkAttributes *attributes,
-					     GtkSourceMark           *mark)
+                                             GtkSourceMark           *mark)
 {
 	gchar *ret;
 
@@ -657,7 +648,7 @@ gtk_source_mark_attributes_get_tooltip_text (GtkSourceMarkAttributes *attributes
  * @mark: a #GtkSourceMark.
  *
  * Queries for a tooltip by emitting
- * a #GtkSourceMarkAttributes::query-tooltip-markup signal. The tooltip may contain
+ * a GtkSourceMarkAttributes::query-tooltip-markup signal. The tooltip may contain
  * a markup.
  *
  * Returns: (transfer full): A tooltip. The returned string should be freed by
@@ -665,7 +656,7 @@ gtk_source_mark_attributes_get_tooltip_text (GtkSourceMarkAttributes *attributes
  */
 gchar *
 gtk_source_mark_attributes_get_tooltip_markup (GtkSourceMarkAttributes *attributes,
-					       GtkSourceMark           *mark)
+                                               GtkSourceMark           *mark)
 {
 	gchar *ret;
 
