@@ -108,6 +108,9 @@ struct _GtkSourceUndoAction
 	guint modified  : 1;
 };
 
+/* INVALID is a pointer to an invalid action */
+#define INVALID ((void *) "IA")
+
 enum
 {
 	INSERT_TEXT,
@@ -138,7 +141,9 @@ struct _GtkSourceUndoManagerDefaultPrivate
 	guint modified_undoing_group : 1;
 
 	/* Pointer to the action (in the action list) marked as "modified".
-	 * It is NULL when no action is marked as "modified". */
+	 * It is NULL when no action is marked as "modified".
+	 * It is INVALID when the action marked as "modified" has been removed
+	 * from the action list (freeing the list or resizing it) */
 	GtkSourceUndoAction *modified_action;
 
 	guint buffer_signals[NUM_SIGNALS];
@@ -183,7 +188,6 @@ static gboolean merge_action          (GtkSourceUndoManagerDefault      *um,
 static void gtk_source_undo_manager_iface_init (GtkSourceUndoManagerIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GtkSourceUndoManagerDefault, gtk_source_undo_manager_default, G_TYPE_OBJECT,
-			 G_ADD_PRIVATE (GtkSourceUndoManagerDefault)
                          G_IMPLEMENT_INTERFACE (GTK_SOURCE_TYPE_UNDO_MANAGER,
                                                 gtk_source_undo_manager_iface_init))
 
@@ -418,12 +422,16 @@ gtk_source_undo_manager_default_class_init (GtkSourceUndoManagerDefaultClass *kl
 	                                                   G_MAXINT,
 	                                                   DEFAULT_MAX_UNDO_LEVELS,
 	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	g_type_class_add_private (object_class, sizeof(GtkSourceUndoManagerDefaultPrivate));
 }
 
 static void
 gtk_source_undo_manager_default_init (GtkSourceUndoManagerDefault *um)
 {
-	um->priv = gtk_source_undo_manager_default_get_instance_private (um);
+	um->priv = G_TYPE_INSTANCE_GET_PRIVATE (um, GTK_SOURCE_TYPE_UNDO_MANAGER_DEFAULT,
+	                                        GtkSourceUndoManagerDefaultPrivate);
+
 	um->priv->actions = g_ptr_array_new ();
 }
 
@@ -502,10 +510,7 @@ insert_text (GtkTextBuffer *buffer,
 	GtkTextIter iter;
 
 	gtk_text_buffer_get_iter_at_offset (buffer, &iter, pos);
-
-	gtk_text_buffer_begin_user_action (buffer);
 	gtk_text_buffer_insert (buffer, &iter, text, len);
-	gtk_text_buffer_end_user_action (buffer);
 }
 
 static void
@@ -523,9 +528,7 @@ delete_text (GtkTextBuffer *buffer,
 	else
 		gtk_text_buffer_get_iter_at_offset (buffer, &end_iter, end);
 
-	gtk_text_buffer_begin_user_action (buffer);
 	gtk_text_buffer_delete (buffer, &start_iter, &end_iter);
-	gtk_text_buffer_end_user_action (buffer);
 }
 
 static gchar *
@@ -782,23 +785,14 @@ static void
 gtk_source_undo_action_free (GtkSourceUndoAction *action)
 {
 	if (action == NULL)
-	{
 		return;
-	}
 
 	if (action->action_type == GTK_SOURCE_UNDO_ACTION_INSERT)
-	{
 		g_free (action->action.insert.text);
-	}
 	else if (action->action_type == GTK_SOURCE_UNDO_ACTION_DELETE)
-	{
 		g_free (action->action.delete.text);
-	}
 	else
-	{
-		g_free (action);
 		g_return_if_reached ();
-	}
 
 	g_free (action);
 }
@@ -816,7 +810,7 @@ free_action_list (GtkSourceUndoManagerDefault *um)
 			--um->priv->num_of_groups;
 
 		if (action->modified)
-			um->priv->modified_action = NULL;
+			um->priv->modified_action = INVALID;
 
 		gtk_source_undo_action_free (action);
 	}
@@ -1002,7 +996,7 @@ free_first_n_actions (GtkSourceUndoManagerDefault *um,
 			--um->priv->num_of_groups;
 
 		if (action->modified)
-			um->priv->modified_action = NULL;
+			um->priv->modified_action = INVALID;
 
 		gtk_source_undo_action_free (action);
 
@@ -1035,7 +1029,7 @@ check_list_size (GtkSourceUndoManagerDefault *um)
 				--um->priv->num_of_groups;
 
 			if (undo_action->modified)
-				um->priv->modified_action = NULL;
+				um->priv->modified_action = INVALID;
 
 			gtk_source_undo_action_free (undo_action);
 
@@ -1195,7 +1189,9 @@ modified_changed_handler (GtkTextBuffer               *buffer,
 
 		if (manager->priv->modified_action != NULL)
 		{
-			manager->priv->modified_action->modified = FALSE;
+			if (manager->priv->modified_action != INVALID)
+				manager->priv->modified_action->modified = FALSE;
+
 			manager->priv->modified_action = NULL;
 		}
 
